<?php

/**
 * @file
 * Contains \Drupal\monitoring\Sensor\SensorManager.
 */

namespace Drupal\monitoring\Sensor;

/**
 * Manages sensor definitions and settings.
 */
class SensorManager {

  /**
   * List of sensor definitions.
   *
   * @var \Drupal\monitoring\Sensor\SensorInfo[]
   */
  protected $info;

  /**
   * Returns monitoring sensor info.
   *
   * @return \Drupal\monitoring\Sensor\SensorInfo[]
   *   List of SensorInfo instances.
   */
  public function getSensorInfo() {
    if (empty($this->info)) {
      $this->info = $this->loadSensorInfo();
    }

    return $this->info;
  }

  /**
   * Returns monitoring sensor info for enabled sensors.
   *
   * @return \Drupal\monitoring\Sensor\SensorInfo[]
   *   List of SensorInfo instances.
   */
  public function getEnabledSensorInfo() {
    $enabled_sensors = array();
    foreach ($this->getSensorInfo() as $sensor_info) {
      if ($sensor_info->isEnabled()) {
        $enabled_sensors[$sensor_info->getName()] = $sensor_info;
      }
    }
    return $enabled_sensors;
  }

  /**
   * Returns monitoring sensor info for a given sensor.
   *
   * @param string $sensor_name
   *   Sensor id.
   *
   * @return \Drupal\monitoring\Sensor\SensorInfo
   *   A single SensorInfo instance.
   *
   * @throws \Drupal\monitoring\Sensor\NonExistingSensorException
   *   Thrown if the requested sensor does not exist.
   */
  public function getSensorInfoByName($sensor_name) {
    $info = $this->getSensorInfo();
    if (isset($info[$sensor_name])) {
      return $info[$sensor_name];
    }
    throw new NonExistingSensorException(format_string('Sensor @sensor_name does not exist', array('@sensor_name' => $sensor_name)));
  }

  /**
   * Gets sensor info grouped by categories.
   *
   * @todo: The enabled flag is strange, FALSE should return all?
   *
   * @param bool $enabled
   *   Sensor isEnabled flag.
   *
   * @return \Drupal\monitoring\Sensor\SensorInfo[]
   *   Sensor info.
   */
  function getSensorInfoByCategories($enabled = TRUE) {
    $info_by_categories = array();
    foreach ($this->getSensorInfo() as $sensor_name => $sensor_info) {
      if ($sensor_info->isEnabled() != $enabled) {
        continue;
      }

      $info_by_categories[$sensor_info->getCategory()][$sensor_name] = $sensor_info;
    }

    // Sort the categories by their name.
    ksort($info_by_categories);

    return $info_by_categories;
  }

  /**
   * Reset the static cache
   */
  public function resetCache() {
    $this->info = array();
  }

  /**
   * Enable a sensor.
   *
   * Checks if the sensor is enabled and enables it if not.
   *
   * @param string $sensor_name
   *   Sensor name to be enabled.
   *
   * @throws \Drupal\monitoring\Sensor\NonExistingSensorException
   *   Thrown if the requested sensor does not exist.
   */
  function enableSensor($sensor_name) {
    $sensor_info = $this->getSensorInfoByName($sensor_name);
    if (!$sensor_info->isEnabled()) {
      $settings = monitoring_sensor_settings_get($sensor_name);
      $settings['enabled'] = TRUE;
      monitoring_sensor_settings_save($sensor_name, $settings);
      // @todo the following part is SensorDisappearedSensors specific. We need
      //   to move it into the sensor somehow.
      $available_sensors = variable_get('monitoring_available_sensors', array());
      $available_sensors[$sensor_name]['enabled'] = TRUE;
      $available_sensors[$sensor_name]['name'] = $sensor_name;
      variable_set('monitoring_available_sensors', $available_sensors);
    }
  }

  /**
   * Disable a sensor.
   *
   * Checks if the sensor is enabled and if so it will disable it and remove
   * from the active sensor list.
   *
   * @param string $sensor_name
   *   Sensor name to be disabled.
   *
   * @throws \Drupal\monitoring\Sensor\NonExistingSensorException
   *   Thrown if the requested sensor does not exist.
   */
  function disableSensor($sensor_name) {
    $sensor_info = $this->getSensorInfoByName($sensor_name);
    if ($sensor_info->isEnabled()) {
      $settings = monitoring_sensor_settings_get($sensor_name);
      $settings['enabled'] = FALSE;
      monitoring_sensor_settings_save($sensor_name, $settings);
      // @todo - the following part is SensorDisappearedSensors specific. We need
      // to move it into the sensor somehow.
      $available_sensors = variable_get('monitoring_available_sensors', array());
      $available_sensors[$sensor_name]['enabled'] = FALSE;
      $available_sensors[$sensor_name]['name'] = $sensor_name;
      variable_set('monitoring_available_sensors', $available_sensors);
    }
  }

  /**
   * Loads sensor info from hooks.
   *
   * @return \Drupal\monitoring\Sensor\SensorInfo[]
   *   List of SensorInfo instances.
   */
  protected function loadSensorInfo() {
    $info = array();
    // A module might provide a separate file with sensor definitions. Try to
    // include it prior to checking if a hook exists.
    // @todo: Use hook_hook_info().
    foreach (module_list() as $module) {
      $sensors_file = drupal_get_path('module', $module) . '/' . $module . '.monitoring_sensors.inc';
      if (file_exists($sensors_file)) {
        require_once $sensors_file;
      }
    }

    // Collect sensors info.
    $custom_implementations = module_implements('monitoring_sensor_info');
    foreach (module_list() as $module) {

      // Favor custom implementation.
      if (in_array($module, $custom_implementations)) {
        $result = module_invoke($module, 'monitoring_sensor_info');
        $info = drupal_array_merge_deep($info, $result);
      }
      // If there is no custom implementation try to find local integration.
      elseif (function_exists('monitoring_' . $module . '_monitoring_sensor_info')) {
        $function = 'monitoring_' . $module . '_monitoring_sensor_info';
        $result = $function();
        if (is_array($result)) {
          $info = drupal_array_merge_deep($info, $result);
        }
      }
    }

    // Allow to alter the collected sensors info.
    drupal_alter('monitoring_sensor_info', $info);

    // Merge in saved sensor settings.
    foreach ($info as $key => &$value) {
      // Set default values.
      $value += array(
        'description' => '',
        'result_class' => 'Drupal\monitoring\Result\SensorResult',
        'numeric' => TRUE,
        'value_label' => NULL,
        'settings' => array(),
      );
      $value['settings'] += array(
        'enabled' => TRUE,
        'caching_time' => 0,
        'category' => 'Other',
      );
      $value['settings'] = $this->mergeSettings($key, $value['settings']);
    }

    // Support variable overrides.
    // @todo This will change in https://drupal.org/node/2170955.
    $info = drupal_array_merge_deep($info, variable_get('monitoring_sensor_info', array()));

    // Convert the arrays into SensorInfo objects.
    foreach ($info as $sensor_name => $sensor_info) {
      $info[$sensor_name] = new SensorInfo($sensor_name, $sensor_info);
    }

    // Sort the sensors by their name.
    ksort($info);

    return $info;
  }

  /**
   * Merges provided sensor settings with saved settings.
   *
   * @param string $sensor_name
   *   Sensor name.
   * @param array $default_settings
   *   Default sensor settings.
   *
   * @return array
   *   Merged settings.
   */
  protected function mergeSettings($sensor_name, array $default_settings) {
    $saved_settings = monitoring_sensor_settings_get($sensor_name);
    $default_settings = $this->mergeSettingsArrays(array($default_settings, $saved_settings));
    return $default_settings;
  }

  /**
   * Merges settings arrays.
   *
   * Based on drupal_array_merge_deep_array() but with the additional special
   * case that flat arrays (arrays that don't have other arrays as values)
   * are replaced, not merged.
   *
   * That allows to expose settings forms for multiple values that can override
   * the default configuration.
   *
   * @param $arrays
   *   List of arrays to merge, later arrays replace keys in the previous.
   *
   * @return array
   *   The merged array.
   */
  protected function mergeSettingsArrays($arrays) {
    $result = array();

    foreach ($arrays as $array) {
      foreach ($array as $key => $value) {
        // Renumber integer keys as array_merge_recursive() does. Note that PHP
        // automatically converts array keys that are integer strings (e.g., '1')
        // to integers.
        if (is_integer($key)) {
          $result[] = $value;
        }
        // Check if we have an array and the first array is flat.
        elseif (isset($result[$key]) && is_array($result[$key]) && is_array($value) && $this->isFlatArray($result[$key]))  {
          $result[$key] = $value;
        }
        // Recurse when both values are arrays.
        elseif (isset($result[$key]) && is_array($result[$key]) && is_array($value)) {
          $result[$key] = $this->mergeSettingsArrays(array($result[$key], $value));
        }
        // Otherwise, use the latter value, overriding any previous value.
        else {
          $result[$key] = $value;
        }
      }
    }
    return $result;
  }

  /**
   * Returns if an array is flat.
   *
   * @param $array
   *   The array to check.
   *
   * @return bool
   *   TRUE if the array has no values that are themself arrays.
   */
  protected function isFlatArray($array) {
    foreach ($array as $value) {
      if (is_array($value)) {
        return FALSE;
      }
    }
    return TRUE;
  }

}
